@@ -83,10 +83,10 @@ BTNCAN = "Cancel"
 BTNRST = "Reset"
 
 # Label text
-LBLPROTDISP = "Protocols Displayed"
+LBLPROTDISP = "Protocols Shown"
 LBLDATADISP = "Console Display"
-LBLUBXCONFIG = "UBX Configuration"
-LBLNTRIPCONFIG = "NTRIP Configuration"
+LBLUBXCONFIG = "UBX\nConfiguration"
+LBLNTRIPCONFIG = "NTRIP\nConfiguration"
 LBLNTRIPGGAINT = "GGA Interval"
 LBLCTL = "Controls"
 LBLSET = "Settings"
@@ -94,9 +94,9 @@ LBLCFGPRT = "CFG-PRT Protocol Configuration"
 LBLCFGRATE = "CFG-RATE Navigation Solution Rate Configuration"
 LBLCFGMSG = "CFG-MSG Message Rate Configuration"
 LBLPRESET = "Preset UBX Configuration Commands"
-LBLDATALOG = "Enable DataLog"
+LBLDATALOG = "DataLogging"
 LBLSTREAM = "Stream\nfrom file"
-LBLTRACKRECORD = "Record GPX Track"
+LBLTRACKRECORD = "GPX Track"
 LBLLEGEND = "Show Legend"
 LBLSHOWUNUSED = "Show Unused Satellites"
 LBLNTRIPSERVER = "Server"
